@@ -1,7 +1,11 @@
 # -*- coding: utf8 -*-
 import weakref
 import unittest
-from weakreflist import WeakList
+from .weakreflist import WeakList
+import sys
+is_pypy = '__pypy__' in sys.builtin_module_names
+if is_pypy:
+    import gc
 
 
 class WeakrefListTest(unittest.TestCase):
@@ -39,6 +43,8 @@ class WeakrefListTest(unittest.TestCase):
         self.wrList.append(myFake)
         self.assertEqual(2, len(self.wrList))
         del myFake
+        if is_pypy:
+            gc.collect()
         self.assertEqual(0, len(self.wrList))
 
     def test_it_can_remove_a_value(self):
@@ -63,20 +69,42 @@ class WeakrefListTest(unittest.TestCase):
         self.assertEqual(0, self.wrList.index(myFake0))
         self.assertEqual(2, self.wrList.index(myFake1))
 
-    def test_it_support_addition(self):
+    def test_it_supports_addition(self):
         myFake = self.objectFake()
         myList = WeakList([myFake])
         self.wrList += myList
         self.assertEqual(1, len(self.wrList))
 
-    def test_it_support_iteration(self):
+    def test_it_supports_iteration(self):
         myFake = self.objectFake()
         self.wrList += [myFake, myFake, myFake, myFake]
         num_mock = 0
         for mock in self.wrList:
             num_mock += 1
             self.assertEqual(mock, myFake)
-        self.assertEqual(num_mock, len(self.wrList))
+        self.assertEqual(num_mock, 4)
+
+    def test_it_appends_ref_values_at_init(self):
+        myFake = self.objectFake()
+        wrList = WeakList([myFake])
+        self.assertEqual(1, len(wrList))
+        del myFake
+        if is_pypy:
+            gc.collect()
+        self.assertEqual(0, len(wrList))
+
+    def test_it_supports_slice_on_int(self):
+        wrList = WeakList(range(10))
+        self.assertEqual(WeakList([1, 2, 3]), wrList[1:4])
+
+    def test_it_supports_slice_on_objects(self):
+        myFake1 = self.objectFake()
+        myFake2 = self.objectFake()
+        myFake3 = self.objectFake()
+        myFake4 = self.objectFake()
+        wrList = WeakList([myFake1, myFake2, myFake3, myFake4])
+        self.assertEqual([myFake2, myFake3], list(wrList[1:3]))
+
 
 
 if __name__ == "__main__":
